/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "eeprom4.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define EEPROM4_DUMMY 0

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void eeprom4_cfg_setup ( eeprom4_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->sck = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs = HAL_PIN_NC;

    // Additional gpio pins

    cfg->wp = HAL_PIN_NC;
    cfg->hld = HAL_PIN_NC;

    cfg->spi_speed = SPI_MEDIUM; 
    cfg->spi_mode = SPI_MODE_0;
}

EEPROM4_RETVAL eeprom4_init ( eeprom4_t *ctx, eeprom4_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->miso;
    spi_cfg.pins.mosi   = cfg->mosi;
    spi_cfg.chip_select = cfg->cs;

    if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
    {
        return EEPROM4_INIT_ERROR;
    }

    spi_master_set_dummy_data( &ctx->spi, EEPROM4_DUMMY );

    // Output pins 
    
    digital_out_init( &ctx->wp, cfg->wp );
    digital_out_init( &ctx->hld, cfg->hld );

    return EEPROM4_OK;

}

void eeprom4_default_cfg ( eeprom4_t *ctx )
{
    eeprom4_enable_write_protect( ctx, EEPROM4_LOGIC_HIGH );
    eeprom4_send_command( ctx, EEPROM4_SET_WRITE_ENABLE_LATCH_COMMAND );
    eeprom4_write_status_reg( ctx, EEPROM4_NONE_PROTECTED_MEMORY_LOCATION );
    eeprom4_enable_hold_operation( ctx, EEPROM4_LOGIC_HIGH );
    
}

void eeprom4_generic_transfer ( eeprom4_t *ctx, spi_master_transfer_data_t *block )
{
    spi_master_start( &ctx->spi );
    spi_master_transfer( &ctx->spi, block );
    spi_master_stop( &ctx->spi );    
}

uint8_t eeprom4_send_command ( eeprom4_t *ctx, uint8_t command_byte )
{
    uint8_t ret_val = 0;

    if ( command_byte == EEPROM4_LOW_POWER_WRITE_POLL_COMMAND )
    {
        spi_master_start( &ctx->spi );

        spi_master_write( &ctx->spi, command_byte );
        ret_val = spi_master_read( &ctx->spi );

        spi_master_stop( &ctx->spi );
        
        return ret_val;
    }
    else
    {
        spi_master_start( &ctx->spi );
        spi_master_write( &ctx->spi, command_byte );
        spi_master_stop( &ctx->spi );  
        
        return 0;
    }
}

void eeprom4_write_status_reg ( eeprom4_t *ctx, uint8_t data_value )
{
    spi_master_start( &ctx->spi );

    spi_master_write( &ctx->spi, 0x01 );
    spi_master_write( &ctx->spi, data_value );

    spi_master_stop( &ctx->spi ); 
}

uint8_t eeprom4_read_status_reg ( eeprom4_t *ctx )
{
    uint8_t data_value = 0;
    
    spi_master_start(&ctx->spi);

    spi_master_write( &ctx->spi, 0x05 );
    data_value = spi_master_read( &ctx->spi );

    spi_master_stop( &ctx->spi ); 

    return data_value;
}

void eeprom4_write_memory ( eeprom4_t *ctx, uint32_t memory_address, uint8_t *data_input, uint8_t n_bytes )
{
    uint8_t cnt = 0;

    spi_master_start( &ctx->spi );
     
    spi_master_write( &ctx->spi, 0x02 );
    spi_master_write( &ctx->spi, ( memory_address >> 16 ) );
    spi_master_write( &ctx->spi, ( memory_address >> 8 ) );
    spi_master_write( &ctx->spi, memory_address );

    for ( cnt = 0; cnt < n_bytes; cnt++ )
    {
        spi_master_write( &ctx->spi, data_input[ cnt ] );
    }

    spi_master_stop( &ctx->spi );    
}

void eeprom4_read_memory ( eeprom4_t *ctx, uint32_t memory_address, uint8_t *data_output, uint8_t n_bytes )
{
    uint8_t cnt = 0;

    spi_master_start( &ctx->spi );

    spi_master_write( &ctx->spi, 0x03 );
    spi_master_write( &ctx->spi, memory_address >> 16 );
    spi_master_write( &ctx->spi, memory_address >> 8 );
    spi_master_write( &ctx->spi, memory_address );
    
    for ( cnt = 0; cnt < n_bytes; cnt++ )
    {
        data_output[ cnt ] = spi_master_read( &ctx->spi );
    }
    
    spi_master_stop( &ctx->spi ); 
    
}

void eeprom4_enable_write_protect ( eeprom4_t *ctx, uint8_t state )
{
    digital_out_write( &ctx->wp, state );
}

void eeprom4_enable_hold_operation ( eeprom4_t *ctx, uint8_t state )
{
    digital_out_write( &ctx->hld, state );
}

uint8_t eeprom4_check_status_reg ( eeprom4_t *ctx, uint8_t check_bit )
{
    uint8_t status_state = 0;
    uint8_t ret_val = 0;
    
    status_state = eeprom4_read_status_reg( ctx );
    
    ret_val = status_state & check_bit;
    
    if ( ( ret_val == 0x01 ) || ( ret_val == 0x02 ) || ( ret_val == 0x08 ) )
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

// ------------------------------------------------------------------------- END

